@@ -14,7 +14,7 @@ use strict;
 use Cache::Memcached;
 use Apache::Session::Generate::MD5;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 # Shared variables
 our $secureTokenMemcachedConnection;
@@ -33,13 +33,13 @@ BEGIN {
 sub run {
     my $class = shift;
     my $r     = $_[0];
-    my $ret   = $class->SUPER::run();
+    my $ret   = $class->Lemonldap::NG::Handler::Main::run($r);
 
     # Continue only if user is authorized
     return $ret unless ( $ret == $class->OK );
 
     # Get current URI
-    my $uri = Lemonldap::NG::Handler::API->uri_with_args($r);
+    my $uri = $r->{env}->{REQUEST_URI};
 
     # Catch Secure Token parameters
     my $localConfig = $class->localConfig;
@@ -94,30 +94,32 @@ sub run {
       unless $class->_isAlive();
 
     # Value to store
-    my $value = $class->datas->{$secureTokenAttribute};
+    my $value = $class->data->{$secureTokenAttribute};
 
     # Set token
     my $key = $class->_setToken( $value, $secureTokenExpiration );
     return $class->_returnError( $r, $secureTokenAllowOnError ) unless $key;
 
     # Header location
-    $class->set_header_in( $secureTokenHeader => $key );
+    $class->set_header_in( $r, $secureTokenHeader => $key );
 
     # Remove token
     eval 'use Apache2::Filter' unless ( $INC{"Apache2/Filter.pm"} );
 
-    $r->add_output_filter(
-        sub {
-            my $f = shift;
-            while ( $f->read( my $buffer, 1024 ) ) {
-                $f->print($buffer);
+    if ( $INC{"Apache2/Filter.pm"} ) {
+        $r->add_output_filter(
+            sub {
+                my $f = shift;
+                while ( $f->read( my $buffer, 1024 ) ) {
+                    $f->print($buffer);
+                }
+                if ( $f->seen_eos ) {
+                    $class->_deleteToken($key);
+                }
+                return $class->OK;
             }
-            if ( $f->seen_eos ) {
-                $class->_deleteToken($key);
-            }
-            return $class->OK;
-        }
-    );
+        );
+    }
 
     # Return $class->OK
     return $class->OK;
@@ -227,7 +229,7 @@ sub _returnError {
     # Redirect or Forbidden?
     if ( $class->tsv->{useRedirectOnError} ) {
         $class->logger->debug("Use redirect for error");
-        return $class->goToPortal( '/', 'lmError=500' );
+        return $class->goToError( '/', 500 );
     }
 
     else {
